/*
 * Factory-related data type macros.
 *
 * Copyright (C) 2019, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <systypes/base.h>



EXTERN_C_BEGIN

/* Type and value constructor macros. */

#define ipc_varg_typedef(value_type, type) \
typedef struct { l4_umword_t tag; value_type value; } type;

#define ipc_varg_mword(type, value) \
((type) {L4_VARG_TYPE_MWORD | (sizeof(l4_mword_t) << 16), value})

#define ipc_varg_umword(type, value) \
((type) {L4_VARG_TYPE_UMWORD | (sizeof(l4_umword_t) << 16), value})

/* Tag inspection macros. */

#define ipc_varg_size(tag) ((tag >> 16) & 0xff)

#define ipc_varg_type(tag) (tag & 0xff)

#define ipc_varg_word(tag) \
((ipc_varg_type(tag) == L4_VARG_TYPE_MWORD) || \
 (ipc_varg_type(tag) == L4_VARG_TYPE_UMWORD))

/* Convenience macro for creating a type name, not necessarily employed by all
   types. */

#define ipc_varg(type) ipc_varg_##type##_t



/* Supporting data types for factory invocations. */

typedef unsigned long unsigned_long;

ipc_varg_typedef(long, ipc_varg(long))
ipc_varg_typedef(unsigned_long, ipc_varg(unsigned_long))

#define ipc_varg_long(value) \
ipc_varg_mword(ipc_varg(long), value)

#define ipc_varg_unsigned_long(value) \
ipc_varg_umword(ipc_varg(unsigned_long), value)



/* Additional, specific factory types for L4 factory interface invocations. */

ipc_varg_typedef(sys_mode_t, ipc_varg_sys_mode_t)
ipc_varg_typedef(sys_uid_t, ipc_varg_sys_uid_t)
ipc_varg_typedef(sys_gid_t, ipc_varg_sys_gid_t)

#define ipc_varg_sys_mode(value) \
ipc_varg_umword(ipc_varg_sys_mode_t, value)

#define ipc_varg_sys_uid(value) \
ipc_varg_umword(ipc_varg_sys_uid_t, value)

#define ipc_varg_sys_gid(value) \
ipc_varg_umword(ipc_varg_sys_gid_t, value)

EXTERN_C_END
